@@ -22,7 +22,6 @@
 #endif
 
 #include "libbluray/bluray.h"
-#include "util/logging.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,13 +46,11 @@ static void _usage(char *cmd)
 int main(int argc, char *argv[])
 {
     BLURAY *bd;
-    int count, ii, opt;
+    int count, ii, opt, main_title;
     unsigned int seconds = 0;
     unsigned int flags = TITLES_RELEVANT;
     char *bd_dir = NULL;
 
-    BD_DEBUG(DBG_BLURAY,"\nListing titles:\n");
-
     do
     {
         opt = getopt(argc, argv, OPTS);
@@ -87,6 +84,10 @@ int main(int argc, char *argv[])
     bd = bd_open(bd_dir, NULL);
 
     count = bd_get_titles(bd, flags, seconds);
+    main_title = bd_get_main_title(bd);
+    if (main_title >= 0) {
+        printf("Main title: %d\n", main_title + 1);
+    }
     for (ii = 0; ii < count; ii++)
     {
         BLURAY_TITLE_INFO* ti;
