@@ -28,7 +28,6 @@
 #include "util/strutl.h"
 
 #include "libbluray/bdnav/mpls_parse.h"
-#include "libbluray/bdnav/navigation.h"
 
 #include "util.h"
 
@@ -120,6 +119,13 @@ const VALUE_MAP subpath_type_map[] = {
   {0,NULL}
 };
 
+const VALUE_MAP playback_type_map[] = {
+  {1, "Sequential"},
+  {2, "Random"},
+  {3, "Shuffle"},
+  {0, NULL}
+};
+
 static const char*
 _lookup_str(const VALUE_MAP *map, int val)
 {
@@ -274,6 +280,17 @@ _show_details(MPLS_PL *pl, int level)
     }
 }
 
+static void
+_show_ai(MPLS_PL *pl, int level)
+{
+    indent_printf(level, "Playback type: %s (%d)",
+                  _lookup_str(playback_type_map, pl->app_info.playback_type),
+                  pl->app_info.playback_type);
+    if (pl->app_info.playback_type == 2 || pl->app_info.playback_type == 3) {
+        indent_printf(level+1, "Playback count: %d", pl->app_info.playback_count);
+    }
+}
+
 static void
 _show_marks(MPLS_PL *pl, int level)
 {
@@ -556,6 +573,7 @@ _process_file(char *name, MPLS_PL *pl_list[], int pl_count)
                     pl->list_count,
                     _pl_duration(pl) / (45000 * 60),
                     (_pl_duration(pl) / 45000) % 60);
+        _show_ai(pl, 1);
     } else {
         indent_printf(0, "%s -- Duration: minutes %4u:%02u",
                     basename(name),
@@ -691,8 +709,6 @@ main(int argc, char *argv[])
         dir = NULL;
         if (S_ISDIR(st.st_mode)) {
 
-            char *main_title = NULL;
-
             printf("Directory: %s:\n", argv[ii]);
             path = str_printf("%s/BDMV/PLAYLIST", argv[ii]);
             if (path == NULL) {
@@ -705,13 +721,6 @@ main(int argc, char *argv[])
                 free(path);
                 continue;
             }
-            main_title = nav_find_main_title(argv[ii]);
-            if (main_title != NULL) {
-                printf("Main Title: %s\n", main_title);
-                free(main_title);
-            } else {
-                fprintf(stderr, "Main title search failed\n");
-            }
         }
         if (dir != NULL) {
             char **dirlist = calloc(10001, sizeof(char*));
