@@ -41,11 +41,13 @@
 #define file_eof(X) X->eof(X)
 #define file_read(X,Y,Z) (size_t)X->read(X,Y,Z)
 #define file_write(X,Y,Z) (size_t)X->write(X,Y,Z)
+BD_PRIVATE int64_t file_size(BD_FILE_H *fp);
 
 BD_PRIVATE extern BD_FILE_H* (*file_open)(const char* filename, const char *mode);
 
 BD_PRIVATE BD_FILE_OPEN file_open_default(void);
 
+
 /*
  * directory access
  */
@@ -55,11 +57,4 @@ BD_PRIVATE BD_FILE_OPEN file_open_default(void);
 
 BD_PRIVATE extern BD_DIR_H* (*dir_open)(const char* dirname);
 
-/*
- * User cache and data dirs
- */
-
-BD_PRIVATE const char *file_get_data_home(void);
-BD_PRIVATE const char *file_get_cache_home(void);
-
 #endif /* FILE_H_ */
