@@ -45,7 +45,6 @@ typedef enum {
 typedef struct bdjava_s BDJAVA;
 
 struct bluray;
-struct bd_registers_s;
 struct indx_root_s;
 struct bd_argb_buffer_s;
 
@@ -53,7 +52,7 @@ typedef void (*bdj_overlay_cb)(struct bluray *, const unsigned *, int, int,
                                int, int, int, int);
 
 BD_PRIVATE BDJAVA* bdj_open(const char *path, struct bluray *bd,
-                            struct bd_registers_s *registers, struct indx_root_s *index,
+                            struct indx_root_s *index,
                             bdj_overlay_cb osd_cb, struct bd_argb_buffer_s *buf);
 BD_PRIVATE int  bdj_start(BDJAVA *bdjava, unsigned title);
 BD_PRIVATE int  bdj_stop(BDJAVA *bdjava);
@@ -61,4 +60,6 @@ BD_PRIVATE void bdj_close(BDJAVA *bdjava);
 BD_PRIVATE int  bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param);
 BD_PRIVATE int  bdj_get_uo_mask(BDJAVA *bdjava);
 
+BD_PRIVATE int  bdj_jvm_available(void); /* 0: no. 1: only jvm. 2: jvm + libbluray.jar. */
+
 #endif
