@@ -40,8 +40,10 @@ public class BDFontMetrics extends FontMetrics {
 
         ftLib = initN();
 
-        if (ftLib == 0)
+        if (ftLib == 0) {
+            System.err.println("freetype library not loaded");
             throw new AWTError("freetype lib not loaded");
+        }
 
         Runtime.getRuntime().addShutdownHook(
             new Thread() {
@@ -51,7 +53,7 @@ public class BDFontMetrics extends FontMetrics {
                         try {
                             BDFontMetrics fm = (BDFontMetrics)it.next();
                             it.remove();
-                            fm.finalize();
+                            fm.destroy();
                         } catch (Throwable e) {
                             e.printStackTrace();
                         }
@@ -241,11 +243,15 @@ public class BDFontMetrics extends FontMetrics {
         return newWidths;
     }
 
-    protected void finalize() throws Throwable {
+    public void destroy() {
         if (ftFace != 0) {
             destroyFontN(ftFace);
             ftFace = 0;
         }
+    }
+
+    protected void finalize() throws Throwable {
+        destroy();
         super.finalize();
     }
 }
