@@ -34,6 +34,7 @@ import org.videolan.BDJAction;
 import org.videolan.BDJActionManager;
 import org.videolan.BDJXletContext;
 import org.videolan.GUIManager;
+import org.videolan.Logger;
 
 public class EventManager implements ResourceServer {
     public static EventManager getInstance() {
@@ -66,7 +67,7 @@ public class EventManager implements ResourceServer {
     }
 
     public void removeUserEventListener(UserEventListener listener) {
-        XletContext context = BDJXletContext.getCurrentContext();
+        BDJXletContext context = BDJXletContext.getCurrentContext();
         synchronized (this) {
             for (Iterator it = sharedUserEventListener.iterator(); it.hasNext(); ) {
                 UserEventItem item = (UserEventItem)it.next();
@@ -97,7 +98,7 @@ public class EventManager implements ResourceServer {
     }
 
     public void removeExclusiveAccessToAWTEvent(ResourceClient client) {
-        XletContext context = BDJXletContext.getCurrentContext();
+        BDJXletContext context = BDJXletContext.getCurrentContext();
         synchronized (this) {
             for (Iterator it = exclusiveAWTEventListener.iterator(); it.hasNext(); ) {
                 UserEventItem item = (UserEventItem)it.next();
@@ -131,24 +132,22 @@ public class EventManager implements ResourceServer {
     }
 
     public boolean receiveKeyEventN(int type, int modifiers, int keyCode) {
+        UserEvent ue = new UserEvent(this, 1, type, keyCode, modifiers, System.currentTimeMillis());
         HScene focusHScene = GUIManager.getInstance().getFocusHScene();
         boolean result = false;
+
         if (focusHScene != null) {
-            XletContext context = focusHScene.getXletContext();
+            BDJXletContext context = focusHScene.getXletContext();
             for (Iterator it = exclusiveAWTEventListener.iterator(); it.hasNext(); ) {
                 UserEventItem item = (UserEventItem)it.next();
+                if (item.context == null) {
+                    it.remove();
+                    continue;
+                }
                 if (item.context == context) {
-                    UserEvent[] evts = item.userEvents.getUserEvent();
-                    for (int i = 0; i < evts.length; i++) {
-                        UserEvent evt = evts[i];
-                        if ((evt.getFamily() == UserEvent.UEF_KEY_EVENT) &&
-                            (evt.getFamily() == UserEvent.UEF_KEY_EVENT) &&
-                            (evt.getCode() == keyCode) &&
-                            (evt.getType() == type)) {
-
-                            result = BDJHelper.postKeyEvent(type, modifiers, keyCode);
-                            return result;
-                        }
+                    if (item.userEvents.contains(ue)) {
+                        result = BDJHelper.postKeyEvent(type, modifiers, keyCode);
+                        return true;
                     }
                 }
             }
@@ -156,16 +155,13 @@ public class EventManager implements ResourceServer {
 
         for (Iterator it = exclusiveUserEventListener.iterator(); it.hasNext(); ) {
             UserEventItem item = (UserEventItem)it.next();
-            UserEvent[] evts = item.userEvents.getUserEvent();
-            for (int i = 0; i < evts.length; i++) {
-                UserEvent evt = evts[i];
-                if ((evt.getFamily() == UserEvent.UEF_KEY_EVENT) &&
-                    (evt.getCode() == keyCode) &&
-                    (evt.getType() == type)) {
-
-                    BDJActionManager.getInstance().putCallback(new UserEventAction(item, i));
-                    return true;
-                }
+            if (item.context == null) {
+                it.remove();
+                continue;
+            }
+            if (item.userEvents.contains(ue)) {
+                item.context.putCallback(new UserEventAction(item, ue));
+                return true;
             }
         }
 
@@ -173,15 +169,13 @@ public class EventManager implements ResourceServer {
 
         for (Iterator it = sharedUserEventListener.iterator(); it.hasNext(); ) {
             UserEventItem item = (UserEventItem)it.next();
-            UserEvent[] evts = item.userEvents.getUserEvent();
-            for (int i = 0; i < evts.length; i++) {
-                UserEvent evt = evts[i];
-                if ((evt.getFamily() == UserEvent.UEF_KEY_EVENT) &&
-                    (evt.getCode() == keyCode) &&
-                    (evt.getType() == type)) {
-                    BDJActionManager.getInstance().putCallback(new UserEventAction(item, i));
-                    result = true;
-                }
+            if (item.context == null) {
+                it.remove();
+                continue;
+            }
+            if (item.userEvents.contains(ue)) {
+                item.context.putCallback(new UserEventAction(item, ue));
+                result = true;
             }
         }
 
@@ -189,14 +183,20 @@ public class EventManager implements ResourceServer {
     }
 
     private boolean cleanupReservedEvents(UserEventRepository userEvents) {
-        XletContext context = BDJXletContext.getCurrentContext();
+        BDJXletContext context = BDJXletContext.getCurrentContext();
         for (Iterator it = exclusiveUserEventListener.iterator(); it.hasNext(); ) {
             UserEventItem item = (UserEventItem)it.next();
                 if (item.context == context)
                     continue;
                 if (hasOverlap(userEvents, item.userEvents)) {
-                    if (!item.client.requestRelease(item.userEvents, null))
+                    try {
+                        if (!item.client.requestRelease(item.userEvents, null))
+                            return false;
+                    } catch (Exception e) {
+                        logger.error("requestRelease() failed: " + e.getClass());
+                        logger.info("" + e.getStackTrace());
                         return false;
+                    }
                     sendResourceStatusEvent(new UserEventAvailableEvent(item.userEvents));
                     it.remove();
                 }
@@ -206,8 +206,14 @@ public class EventManager implements ResourceServer {
             if (item.context == context)
                 continue;
             if (hasOverlap(userEvents, item.userEvents)) {
-                if (!item.client.requestRelease(item.userEvents, null))
+                try {
+                    if (!item.client.requestRelease(item.userEvents, null))
+                        return false;
+                } catch (Exception e) {
+                    logger.error("requestRelease() failed: " + e.getClass());
+                    logger.info("" + e.getStackTrace());
                     return false;
+                }
                 sendResourceStatusEvent(new UserEventAvailableEvent(item.userEvents));
                 it.remove();
             }
@@ -230,33 +236,35 @@ public class EventManager implements ResourceServer {
     }
 
     private class UserEventItem {
-        public UserEventItem(XletContext context, UserEventListener listener,
+        public UserEventItem(BDJXletContext context, UserEventListener listener,
                              ResourceClient client, UserEventRepository userEvents) {
             this.context = context;
             this.listener = listener;
             this.client = client;
-            this.userEvents = userEvents;
+            this.userEvents = userEvents.getNewInstance();
+            if (context == null) {
+                logger.error("Missing xlet context: " + Logger.dumpStack());
+            }
         }
 
-        public XletContext context;
+        public BDJXletContext context;
         public UserEventListener listener;
         public ResourceClient client;
         public UserEventRepository userEvents;
     }
 
     private class UserEventAction extends BDJAction {
-        public UserEventAction(UserEventItem item, int event) {
-            super((BDJXletContext)item.context);
-            this.item = item;
+        public UserEventAction(UserEventItem item, UserEvent event) {
+            this.listener = item.listener;
             this.event = event;
         }
 
         protected void doAction() {
-            item.listener.userEventReceived(item.userEvents.getUserEvent()[event]);
+            listener.userEventReceived(event);
         }
 
-        private UserEventItem item;
-        private int event;
+        private UserEventListener listener;
+        private UserEvent event;
     }
 
     private LinkedList exclusiveUserEventListener = new LinkedList();
@@ -265,4 +273,6 @@ public class EventManager implements ResourceServer {
     private LinkedList resourceStatusEventListeners = new LinkedList();
 
     private static EventManager instance = null;
+
+    private static final Logger logger = Logger.getLogger(EventManager.class.getName());
 }
