@@ -27,6 +27,7 @@ import java.security.PrivilegedAction;
 
 import javax.microedition.xlet.UnavailableContainerException;
 
+import org.bluray.ui.FrameAccurateAnimation;
 import org.dvb.application.AppID;
 import org.dvb.application.AppProxy;
 import org.dvb.application.AppsDatabase;
@@ -49,6 +50,8 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
                                               Integer.toHexString(appid.getAID()) + "." +
                                               entry.getInitialClass(),
                                               this);
+
+        callbackQueue = new BDJActionQueue(this.threadGroup);
     }
 
     public Object getXletProperty(String key) {
@@ -114,6 +117,15 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
         return eventQueue;
     }
 
+    public boolean putCallback(BDJAction cb)
+    {
+        if (!isDestroyed()) {
+            callbackQueue.put(cb);
+            return true;
+        }
+        return false;
+    }
+
     protected int numEventQueueThreads() {
         int cnt = 0;
         if (eventQueue != null) {
@@ -163,6 +175,29 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
         }
     }
 
+    public void addFAA(FrameAccurateAnimation faa) {
+        synchronized (faaList) {
+            faaList.add(faa);
+        }
+    }
+
+    public void removeFAA(FrameAccurateAnimation faa) {
+        synchronized (faaList) {
+            faaList.remove(faa);
+        }
+    }
+
+    public void removeAllFAA() {
+        Object[] faaArray;
+        synchronized (faaList) {
+            faaArray = faaList.toArray();
+        }
+        for (int i = 0; i < faaArray.length; i++) {
+            FrameAccurateAnimation faa = (FrameAccurateAnimation)faaArray[i];
+            faa.destroy();
+        }
+    }
+
     public static BDJXletContext getCurrentContext() {
         Object obj = AccessController.doPrivileged(
                 new PrivilegedAction() {
@@ -193,6 +228,8 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
     }
 
     protected void release() {
+
+        removeAllFAA();
         stopIxcThreads();
 
         org.dvb.io.ixc.IxcRegistry.unbindAll(this);
@@ -202,6 +239,13 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
             sceneFactory = null;
         }
 
+        try {
+            callbackQueue.finalize();
+        } catch (Throwable t) {
+        } finally {
+            //callbackQueue = null;
+        }
+
         EventQueue eq = eventQueue;
         eventQueue = null;
         if (eq != null) {
@@ -223,5 +267,7 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
     private HSceneFactory sceneFactory = null;
     private BDJThreadGroup threadGroup = null;
     private LinkedList ixcThreads = new LinkedList();
+    private LinkedList faaList = new LinkedList();
+    private BDJActionQueue callbackQueue;
     private static final Logger logger = Logger.getLogger(BDJXletContext.class.getName());
 }
