@@ -46,8 +46,8 @@ public class AppEntry implements AppAttributes {
 
     public String getName(String language) throws LanguageNotAvailableException {
         for (int i = 0; i < names.length; i++)
-                if (language.equals(names[i][0]))
-                        return names[i][1];
+            if (language.equals(names[i][0]))
+                return names[i][1];
         throw new LanguageNotAvailableException();
     }
 
@@ -74,24 +74,24 @@ public class AppEntry implements AppAttributes {
         if (name.equals("dvb.j.location.base"))
             return basePath;
         if (name.equals("dvb.j.location.cpath.extension"))
-                return StrUtil.split(classpathExt, ';');
+            return StrUtil.split(classpathExt, ';');
         if (name.equals("dvb.transport.oc.component.tag"))
-                return null;
+            return null;
         return null;
     }
 
     public Locator getServiceLocator() {
         /*
-         try {
-      String discID = BDJTitleInfoHelper.getCurrent32LengthDiscIDFromC();
-      int titleNumber = BDJTitleInfoHelper.getCurrentTitleNumberFromC();
-      return new BDLocator(discID, titleNumber, 0);
-    }
-    catch (InvalidLocatorException e) {
-      BDJTrace.printStack(e.getStackTrace());
-      BDJAssert.doAssert(false, "can not get the service locator");
-    }return null;
-         */
+        try {
+            String discID = BDJTitleInfoHelper.getCurrent32LengthDiscIDFromC();
+            int titleNumber = BDJTitleInfoHelper.getCurrentTitleNumberFromC();
+            return new BDLocator(discID, titleNumber, 0);
+        }
+        catch (InvalidLocatorException e) {
+            BDJTrace.printStack(e.getStackTrace());
+            BDJAssert.doAssert(false, "can not get the service locator");
+        }
+        */
         return null;
     }
 
@@ -101,13 +101,13 @@ public class AppEntry implements AppAttributes {
 
     public int[] getVersions(String profile) throws IllegalProfileParameterException {
         if (profile.equals("mhp.profile.enhanced_broadcast")) {
-                for (int i = 0; i < profiles.length; i++)
+            for (int i = 0; i < profiles.length; i++)
                 if (profiles[i].getProfile() == 2)
                     return new int[] { profiles[i].getMajor(), profiles[i].getMinor(), profiles[i].getMicro() };
             return null;
-    }
+        }
         if (profile.equals("mhp.profile.interactive_broadcast")) {
-                for (int i = 0; i < profiles.length; i++)
+            for (int i = 0; i < profiles.length; i++)
                 if (profiles[i].getProfile() == 1)
                     return new int[] { profiles[i].getMajor(), profiles[i].getMinor(), profiles[i].getMicro() };
             return null;
@@ -123,15 +123,15 @@ public class AppEntry implements AppAttributes {
         return ((visibility & VISIBLE_TO_USERS) != 0);
     }
 
-        public boolean isDiscBound() {
-                return ((binding & DISC_BOUND) != 0);
-        }
+    public boolean isDiscBound() {
+        return ((binding & DISC_BOUND) != 0);
+    }
 
-        public AppEntry(int controlCode, int type, int orgId,
-            short appId, AppProfile[] profiles, short priority,
-            int binding, int visibility, String[][] names,
-            String iconLocator, short iconFlags, String basePath,
-            String classpathExt, String initialClass, String[] params) {
+    public AppEntry(int controlCode, int type, int orgId,
+                    short appId, AppProfile[] profiles, short priority,
+                    int binding, int visibility, String[][] names,
+                    String iconLocator, short iconFlags, String basePath,
+                    String classpathExt, String initialClass, String[] params) {
         this.controlCode = controlCode;
         this.type = type;
         this.appid = new AppID(orgId, appId);
@@ -141,17 +141,18 @@ public class AppEntry implements AppAttributes {
         this.binding = binding;
         this.visibility = visibility;
         this.names = names;
-        if ((iconLocator != null) && (iconLocator.length() > 0))
-                try {
-                        BDLocator locator = new BDLocator("bd://JAR:" + basePath + "/" + iconLocator);
-                        BitSet flags = new BitSet(16);
+        if ((iconLocator != null) && (iconLocator.length() > 0)) {
+            try {
+                BDLocator locator = new BDLocator("bd://JAR:" + basePath + "/" + iconLocator);
+                BitSet flags = new BitSet(16);
                 for (int i = 0; i < 9; i++)
                     if ((iconFlags & (1 << i)) != 0)
                         flags.set(i, true);
                 this.icon = new AppIcon(locator, flags);
-                } catch (Exception e) {
-                        e.printStackTrace();
-                }
+            } catch (Exception e) {
+                e.printStackTrace();
+            }
+        }
         this.basePath = basePath;
         this.classpathExt = classpathExt;
         this.initialClass = initialClass;
