@@ -1,6 +1,7 @@
 /*
  * This file is part of libbluray
  * Copyright (C) 2009-2010  John Stebbins
+ * Copyright (C) 2012  Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +21,7 @@
 #include "util/macro.h"
 #include "file/file.h"
 #include "util/bits.h"
+#include "extdata_parse.h"
 #include "mpls_parse.h"
 
 #include <stdlib.h>
@@ -29,8 +31,6 @@
 #define MPLS_SIG2A ('0' << 24 | '2' << 16 | '0' << 8 | '0')
 #define MPLS_SIG2B ('0' << 24 | '1' << 16 | '0' << 8 | '0')
 
-static int mpls_verbose = 0;
-
 static void
 _human_readable_sig(char *sig, uint32_t s1, uint32_t s2)
 {
@@ -101,7 +101,7 @@ static int
 _parse_appinfo(BITSTREAM *bits, MPLS_AI *ai)
 {
     int len;
-    int pos;
+    off_t pos;
 
     if (!bs_is_align(bits, 0x07)) {
         fprintf(stderr, "_parse_appinfo: alignment error\n");
@@ -161,7 +161,7 @@ static int
 _parse_stream(BITSTREAM *bits, MPLS_STREAM *s)
 {
     int len;
-    int pos;
+    off_t pos;
 
     if (!bs_is_align(bits, 0x07)) {
         fprintf(stderr, "_parse_stream: Stream alignment error\n");
@@ -248,7 +248,7 @@ static int
 _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
 {
     int len;
-    int pos;
+    off_t pos;
     MPLS_STREAM    *ss;
     int ii,jj;
 
@@ -421,7 +421,7 @@ static int
 _parse_playitem(BITSTREAM *bits, MPLS_PI *pi)
 {
     int len, ii;
-    int pos;
+    off_t pos;
     char clip_id[6], codec_id[5];
     uint8_t stc_id;
 
@@ -515,7 +515,7 @@ static int
 _parse_subplayitem(BITSTREAM *bits, MPLS_SUB_PI *spi)
 {
     int len, ii;
-    int pos;
+    off_t pos;
     char clip_id[6], codec_id[5];
     uint8_t stc_id;
 
@@ -594,7 +594,7 @@ static int
 _parse_subpath(BITSTREAM *bits, MPLS_SUB *sp)
 {
     int len, ii;
-    int pos;
+    off_t pos;
     MPLS_SUB_PI *spi = NULL;
 
     if (!bs_is_align(bits, 0x07)) {
@@ -724,6 +724,12 @@ _clean_playlist(MPLS_PL *pl)
         }
         X_FREE(pl->sub_path);
     }
+    if (pl->ext_sub_path != NULL) {
+        for (ii = 0; ii < pl->ext_sub_count; ii++) {
+            _clean_subpath(&pl->ext_sub_path[ii]);
+        }
+        X_FREE(pl->ext_sub_path);
+    }
     X_FREE(pl->play_mark);
     X_FREE(pl);
 }
@@ -734,15 +740,150 @@ mpls_free(MPLS_PL *pl)
     _clean_playlist(pl);
 }
 
+static int
+_parse_pip_data(BITSTREAM *bits, MPLS_PIP_METADATA *block)
+{
+    MPLS_PIP_DATA *data;
+    unsigned ii;
+
+    uint16_t entries = bs_read(bits, 16);
+    if (entries < 1) {
+        return 1;
+    }
+
+    data = calloc(entries, sizeof(MPLS_PIP_DATA));
+    for (ii = 0; ii < entries; ii++) {
+
+        data[ii].time = bs_read(bits, 32);
+        data[ii].xpos = bs_read(bits, 12);
+        data[ii].ypos = bs_read(bits, 12);
+        data[ii].scale_factor = bs_read(bits, 4);
+        bs_skip(bits, 4);
+    }
+
+    block->data_count = entries;
+    block->data = data;
+
+    return 1;
+}
+
+static int
+_parse_pip_metadata_block(BITSTREAM *bits, uint32_t start_address, MPLS_PIP_METADATA *data)
+{
+    uint32_t data_address;
+    int result;
+    off_t pos;
+
+    data->clip_ref            = bs_read(bits, 16);
+    data->secondary_video_ref = bs_read(bits, 8);
+    bs_skip(bits, 8);
+    data->timeline_type       = bs_read(bits, 4);
+    data->luma_key_flag       = bs_read(bits, 1);
+    data->trick_play_flag     = bs_read(bits, 1);
+    bs_skip(bits, 10);
+    if (data->luma_key_flag) {
+        bs_skip(bits, 8);
+        data->upper_limit_luma_key = bs_read(bits, 8);
+    } else {
+        bs_skip(bits, 16);
+    }
+    bs_skip(bits, 16);
+
+    data_address = bs_read(bits, 32);
+
+    pos = bs_pos(bits) / 8;
+    bs_seek_byte(bits, start_address + data_address);
+    result = _parse_pip_data(bits, data);
+    bs_seek_byte(bits, pos);
+
+    return result;
+}
+
+static int
+_parse_pip_metadata_extension(BITSTREAM *bits, MPLS_PL *pl)
+{
+    MPLS_PIP_METADATA *data;
+    int ii;
+
+    uint32_t start_address = (uint32_t)bs_pos(bits) / 8;
+    uint32_t len           = bs_read(bits, 32);
+    int      entries       = bs_read(bits, 16);
+
+    if (len < 1 || entries < 1) {
+        return 0;
+    }
+
+    data = calloc(entries, sizeof(MPLS_PIP_METADATA));
+    for (ii = 0; ii < entries; ii++) {
+      if (!_parse_pip_metadata_block(bits, start_address, data)) {
+            X_FREE(data);
+            fprintf(stderr, "error parsing pip metadata extension\n");
+            return 0;
+        }
+    }
+
+    pl->ext_pip_data_count = entries;
+    pl->ext_pip_data       = data;
+
+    return 1;
+}
+
+static int
+_parse_subpath_extension(BITSTREAM *bits, MPLS_PL *pl)
+{
+    MPLS_SUB *sub_path;
+    int ii;
+
+    uint32_t len       = bs_read(bits, 32);
+    int      sub_count = bs_read(bits, 16);
+
+    if (len < 1 || sub_count < 1) {
+        return 0;
+    }
+
+    sub_path = calloc(sub_count,  sizeof(MPLS_SUB));
+    for (ii = 0; ii < sub_count; ii++) {
+        if (!_parse_subpath(bits, &sub_path[ii])) {
+            X_FREE(sub_path);
+            fprintf(stderr, "error parsing extension subpath\n");
+            return 0;
+        }
+    }
+    pl->ext_sub_path  = sub_path;
+    pl->ext_sub_count = sub_count;
+
+    return 1;
+}
+
+static int
+_parse_mpls_extension(BITSTREAM *bits, int id1, int id2, void *handle)
+{
+    MPLS_PL *pl = (MPLS_PL*)handle;
+
+    if (id1 == 1) {
+        if (id2 == 1) {
+            // PiP metadata extension
+            return _parse_pip_metadata_extension(bits, pl);
+        }
+    }
+
+    if (id1 == 2) {
+        if (id2 == 2) {
+            // SubPath entries extension
+            return _parse_subpath_extension(bits, pl);
+        }
+    }
+
+    return 0;
+}
+
 static MPLS_PL*
-_mpls_parse(const char *path, int verbose)
+_mpls_parse(const char *path)
 {
     BITSTREAM  bits;
     BD_FILE_H *fp;
     MPLS_PL   *pl = NULL;
 
-    mpls_verbose = verbose;
-
     pl = calloc(1, sizeof(MPLS_PL));
     if (pl == NULL) {
         return NULL;
@@ -771,25 +912,32 @@ _mpls_parse(const char *path, int verbose)
         _clean_playlist(pl);
         return NULL;
     }
+    if (pl->ext_pos > 0) {
+        bdmv_parse_extension_data(&bits,
+                                  pl->ext_pos,
+                                  _parse_mpls_extension,
+                                  pl);
+    }
+
     file_close(fp);
     return pl;
 }
 
 MPLS_PL*
-mpls_parse(const char *path, int verbose)
+mpls_parse(const char *path)
 {
-    MPLS_PL *pl = _mpls_parse(path, verbose);
+    MPLS_PL *pl = _mpls_parse(path);
 
     /* if failed, try backup file */
     if (!pl) {
-        int   len    = strlen(path);
+        size_t len   = strlen(path);
         char *backup = malloc(len + 8);
 
         strncpy(backup, path, len - 19);
         strcpy(backup + len - 19, "BACKUP/");
         strcpy(backup + len - 19 + 7, path + len - 19);
 
-        pl = _mpls_parse(backup, verbose);
+        pl = _mpls_parse(backup);
 
         X_FREE(backup);
     }
