@@ -38,6 +38,7 @@
 #include "bdnav/sound_parse.h"
 #include "hdmv/hdmv_vm.h"
 #include "decoders/graphics_controller.h"
+#include "decoders/m2ts_filter.h"
 #include "decoders/overlay.h"
 #include "file/file.h"
 #ifdef USING_BDJAVA
@@ -96,6 +97,8 @@ typedef struct {
     uint16_t        ig_pid; /* pid of currently selected IG stream */
     uint16_t        pg_pid; /* pid of currently selected PG stream */
 
+    M2TS_FILTER    *m2ts_filter;
+
     /* BD+ */
     BD_BDPLUS_ST   *bdplus;
 } BD_STREAM;
@@ -156,6 +159,9 @@ struct bluray {
 #ifdef USING_BDJAVA
     BDJAVA         *bdjava;
 #endif
+    /* delayed sending of BDJ_EVENT_END_OF_PLAYLIST:
+     * 1 - message pending. 3 - message sent. */
+    uint8_t         bdj_end_of_playlist;
 
     /* HDMV graphics */
     GRAPHICS_CONTROLLER *graphics_controller;
@@ -501,6 +507,8 @@ static void _close_m2ts(BD_STREAM *st)
         st->fp = NULL;
     }
 
+    m2ts_filter_close(&st->m2ts_filter);
+
     libbdplus_m2ts_close(&st->bdplus);
 
     /* reset UO mask */
@@ -518,22 +526,36 @@ static int _open_m2ts(BLURAY *bd, BD_STREAM *st)
     st->fp = file_open(f_name, "rb");
     X_FREE(f_name);
 
+    st->clip_size = 0;
     st->clip_pos = (uint64_t)st->clip->start_pkt * 192;
     st->clip_block_pos = (st->clip_pos / 6144) * 6144;
 
     if (st->fp) {
-        file_seek(st->fp, 0, SEEK_END);
-        if ((st->clip_size = file_tell(st->fp))) {
-            file_seek(st->fp, st->clip_block_pos, SEEK_SET);
+        int64_t clip_size = file_size(st->fp);
+        if (clip_size > 0) {
+
+            if (file_seek(st->fp, st->clip_block_pos, SEEK_SET) < 0) {
+                BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Unable to seek clip %s!\n", st->clip->name);
+                _close_m2ts(st);
+                return 0;
+            }
+
+            st->clip_size   = clip_size;
             st->int_buf_off = 6144;
 
             libaacs_select_title(bd->libaacs, bd_psr_read(bd->regs, PSR_TITLE_NUMBER));
 
             if (st == &bd->st0) {
                 MPLS_PL *pl = st->clip->title->pl;
+                MPLS_STN *stn = &pl->play_item[st->clip->ref].stn;
+
                 st->uo_mask = bd_uo_mask_combine(pl->app_info.uo_mask,
                                                  pl->play_item[st->clip->ref].uo_mask);
 
+                st->m2ts_filter = m2ts_filter_init((int64_t)st->clip->in_time << 1,
+                                                   (int64_t)st->clip->out_time << 1,
+                                                   stn->num_video, stn->num_audio);
+
                 _update_clip_psrs(bd, st->clip);
 
                 _init_pg_stream(bd);
@@ -547,6 +569,7 @@ static int _open_m2ts(BLURAY *bd, BD_STREAM *st)
         }
 
         BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Clip %s empty!\n", st->clip->name);
+        _close_m2ts(st);
     }
 
     BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Unable to open clip %s!\n", st->clip->name);
@@ -559,21 +582,21 @@ static int _read_block(BLURAY *bd, BD_STREAM *st, uint8_t *buf)
     const size_t len = 6144;
 
     if (st->fp) {
-        BD_DEBUG(DBG_STREAM, "Reading unit [%zd bytes] at %"PRIu64"...\n",
-              len, st->clip_block_pos);
+        BD_DEBUG(DBG_STREAM, "Reading unit at %"PRIu64"...\n", st->clip_block_pos);
 
         if (len + st->clip_block_pos <= st->clip_size) {
             size_t read_len;
 
             if ((read_len = file_read(st->fp, buf, len))) {
-                if (read_len != len)
-                    BD_DEBUG(DBG_STREAM | DBG_CRIT, "Read %zd bytes at %"PRIu64" ; requested %zd !\n", read_len, st->clip_block_pos, len);
-
+                if (read_len != len) {
+                    BD_DEBUG(DBG_STREAM | DBG_CRIT, "Read %d bytes at %"PRIu64" ; requested %d !\n", (int)read_len, st->clip_block_pos, (int)len);
+                }
                 if (bd->libaacs && libaacs_decrypt_unit(bd->libaacs, buf)) {
+                    _queue_event(bd, BD_EVENT_ENCRYPTED, BD_ERROR_AACS);
                     return -1;
                 }
-                if (st->bdplus && libbdplus_fixup(st->bdplus, buf, len)) {
-                    return -1;
+                if (st->bdplus && (libbdplus_fixup(st->bdplus, buf, len) < 0)) {
+                    _queue_event(bd, BD_EVENT_ENCRYPTED, BD_ERROR_BDPLUS);
                 }
 
                 st->clip_block_pos += len;
@@ -584,11 +607,19 @@ static int _read_block(BLURAY *bd, BD_STREAM *st, uint8_t *buf)
                     if (buf[4] != 0x47 || buf[4+192] != 0x47 || buf[4+2*192] != 0x47) {
                         BD_DEBUG(DBG_BLURAY | DBG_CRIT,
                                  "TP header copy permission indicator != 0, unit is still encrypted?\n");
-                        _queue_event(bd, BD_EVENT_ENCRYPTED, 0);
+                        _queue_event(bd, BD_EVENT_ENCRYPTED, BD_ERROR_AACS);
                         return -1;
                     }
                 }
 
+                if (st->m2ts_filter) {
+                    int result = m2ts_filter(st->m2ts_filter, buf);
+                    if (result < 0) {
+                        m2ts_filter_close(&st->m2ts_filter);
+                        BD_DEBUG(DBG_BLURAY | DBG_CRIT, "m2ts filter error\n");
+                    }
+                }
+
                 BD_DEBUG(DBG_STREAM, "Read unit OK!\n");
 
 #ifdef BLURAY_READ_ERROR_TEST
@@ -599,14 +630,17 @@ static int _read_block(BLURAY *bd, BD_STREAM *st, uint8_t *buf)
 #endif
             }
 
-            BD_DEBUG(DBG_STREAM | DBG_CRIT, "Read %zd bytes at %"PRIu64" failed !\n", len, st->clip_block_pos);
+            BD_DEBUG(DBG_STREAM | DBG_CRIT, "Read unit at %"PRIu64" failed !\n", st->clip_block_pos);
 
             _queue_event(bd, BD_EVENT_READ_ERROR, 0);
 
             /* skip broken unit */
             st->clip_block_pos += len;
             st->clip_pos += len;
-            file_seek(st->fp, st->clip_block_pos, SEEK_SET);
+
+            if (file_seek(st->fp, st->clip_block_pos, SEEK_SET) < 0) {
+                BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Unable to seek clip %s!\n", st->clip->name);
+            }
 
             return 0;
         }
@@ -702,10 +736,16 @@ static int64_t _seek_stream(BLURAY *bd, BD_STREAM *st,
         }
     }
 
+    if (st->m2ts_filter) {
+        m2ts_filter_seek(st->m2ts_filter, 0, (int64_t)st->clip->in_time << 1);
+    }
+
     st->clip_pos = (uint64_t)clip_pkt * 192;
     st->clip_block_pos = (st->clip_pos / 6144) * 6144;
 
-    file_seek(st->fp, st->clip_block_pos, SEEK_SET);
+    if (file_seek(st->fp, st->clip_block_pos, SEEK_SET) < 0) {
+        BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Unable to seek clip %s!\n", st->clip->name);
+    }
 
     st->int_buf_off = 6144;
 
@@ -765,6 +805,7 @@ static int _run_gc(BLURAY *bd, gc_ctrl_e msg, uint32_t param)
 static int _libaacs_init(BLURAY *bd, const char *keyfile_path)
 {
     int result;
+    const uint8_t *disc_id;
 
     libaacs_unload(&bd->libaacs);
 
@@ -786,8 +827,9 @@ static int _libaacs_init(BLURAY *bd, const char *keyfile_path)
     bd->disc_info.aacs_error_code = result;
     bd->disc_info.aacs_handled    = !result;
     bd->disc_info.aacs_mkbv       = libaacs_get_mkbv(bd->libaacs);
-    if (libaacs_get_disc_id(bd->libaacs)) {
-        memcpy(bd->disc_info.disc_id, libaacs_get_disc_id(bd->libaacs), 20);
+    disc_id = libaacs_get_aacs_data(bd->libaacs, BD_AACS_DISC_ID);
+    if (disc_id) {
+        memcpy(bd->disc_info.disc_id, disc_id, 20);
     }
 
     if (result) {
@@ -800,25 +842,12 @@ static int _libaacs_init(BLURAY *bd, const char *keyfile_path)
     return 1;
 }
 
-const uint8_t *bd_get_vid(BLURAY *bd)
+const uint8_t *bd_get_aacs_data(BLURAY *bd, int type)
 {
     /* internal function. Used by BD-J and libbdplus loader. */
-    return libaacs_get_vid(bd->libaacs);
+    return libaacs_get_aacs_data(bd->libaacs, type);
 }
 
-#ifdef USING_BDJAVA
-const uint8_t *bd_get_pmsn(BLURAY *bd)
-{
-    return libaacs_get_pmsn(bd->libaacs);
-}
-
-const uint8_t *bd_get_device_binding_id(BLURAY *bd)
-{
-    /* internal function. Used by BD-J. */
-    return libaacs_get_device_binding_id(bd->libaacs);
-}
-#endif /* USING_BDJAVA */
-
 static int _libbdplus_init(BLURAY *bd)
 {
     libbdplus_unload(&bd->libbdplus);
@@ -834,7 +863,15 @@ static int _libbdplus_init(BLURAY *bd)
         return 0;
     }
 
-    if (libbdplus_init(bd->libbdplus, bd->device_path, bd_get_vid(bd))) {
+    const uint8_t *vid = libaacs_get_aacs_data(bd->libaacs, BD_AACS_MEDIA_VID);
+    const uint8_t *mk  = libaacs_get_aacs_data(bd->libaacs, BD_AACS_MEDIA_KEY);
+    if (!vid) {
+        BD_DEBUG(DBG_BLURAY | DBG_CRIT, "BD+ initialization failed (no AACS ?)\n");
+        libaacs_unload(&bd->libaacs);
+        return 0;
+    }
+
+    if (libbdplus_init(bd->libbdplus, bd->device_path, vid, mk)) {
         BD_DEBUG(DBG_BLURAY | DBG_CRIT, "bdplus_init() failed\n");
 
         bd->disc_info.bdplus_handled = 0;
@@ -851,9 +888,8 @@ static int _libbdplus_init(BLURAY *bd)
     /* connect registers */
     libbdplus_psr(bd->libbdplus, (void*)bd->regs, (void*)bd_psr_read, (void*)bd_psr_write);
 
-    /* start VM */
-    libbdplus_start(bd->libbdplus);
-
+    bd->disc_info.bdplus_gen     = libbdplus_get_gen(bd->libbdplus);
+    bd->disc_info.bdplus_date    = libbdplus_get_date(bd->libbdplus);
     bd->disc_info.bdplus_handled = 1;
     return 1;
 }
@@ -887,6 +923,7 @@ static int _meta_open(BLURAY *bd)
 
     return !!bd->meta;
 }
+
 /*
  * disc info
  */
@@ -905,12 +942,51 @@ static void _fill_disc_info(BLURAY *bd)
     bd->disc_info.num_bdj_titles         = 0;
     bd->disc_info.num_unsupported_titles = 0;
 
+    bd->disc_info.bdj_detected    = 0;
+    bd->disc_info.bdj_supported   = 0;
+    bd->disc_info.libjvm_detected = 0;
+    bd->disc_info.bdj_handled     = 0;
+
     if (bd->index) {
         INDX_PLAY_ITEM *pi;
         unsigned        ii;
 
         bd->disc_info.bluray_detected = 1;
 
+        /* count titles */
+
+        for (ii = 0; ii < bd->index->num_titles; ii++) {
+            if (bd->index->titles[ii].object_type == indx_object_type_hdmv) {
+                bd->disc_info.num_hdmv_titles++;
+            }
+            if (bd->index->titles[ii].object_type == indx_object_type_bdj) {
+                bd->disc_info.num_bdj_titles++;
+                bd->disc_info.num_unsupported_titles++;
+                bd->disc_info.bdj_detected = 1;
+            }
+        }
+
+        if (bd->index->first_play.object_type == indx_object_type_bdj) {
+            bd->disc_info.bdj_detected = 1;
+        }
+        if (bd->index->top_menu.object_type == indx_object_type_bdj) {
+            bd->disc_info.bdj_detected = 1;
+        }
+
+        /* BD-J capability */
+
+#ifdef USING_BDJAVA
+        if (bd->disc_info.bdj_detected) {
+            bd->disc_info.bdj_supported = 1;
+            /* BD-J titles found. Check if jvm + jar can be loaded ? */
+            switch (bdj_jvm_available()) {
+                case 2: bd->disc_info.bdj_handled     = 1;
+                case 1: bd->disc_info.libjvm_detected = 1;
+                default:;
+            }
+        }
+#endif /* USING_BDJAVA */
+
         pi = &bd->index->first_play;
         if (pi->object_type == indx_object_type_hdmv && pi->hdmv.id_ref != 0xffff) {
             bd->disc_info.first_play_supported = 1;
@@ -920,16 +996,6 @@ static void _fill_disc_info(BLURAY *bd)
         if (pi->object_type == indx_object_type_hdmv && pi->hdmv.id_ref != 0xffff) {
             bd->disc_info.top_menu_supported = 1;
         }
-
-        for (ii = 0; ii < bd->index->num_titles; ii++) {
-            if (bd->index->titles[ii].object_type == indx_object_type_hdmv) {
-                bd->disc_info.num_hdmv_titles++;
-            }
-            if (bd->index->titles[ii].object_type == indx_object_type_bdj) {
-                bd->disc_info.num_bdj_titles++;
-                bd->disc_info.num_unsupported_titles++;
-            }
-        }
     }
 }
 
@@ -954,6 +1020,36 @@ uint64_t bd_get_uo_mask(BLURAY *bd)
 }
 #endif
 
+#ifdef USING_BDJAVA
+uint32_t bd_reg_read(BLURAY *bd, int psr, int reg)
+{
+    if (psr) {
+        return bd_psr_read(bd->regs, reg);
+    } else {
+        return bd_gpr_read(bd->regs, reg);
+    }
+}
+#endif
+
+#ifdef USING_BDJAVA
+int bd_reg_write(BLURAY *bd, int psr, int reg, uint32_t value)
+{
+    if (psr) {
+        if (psr < 102) {
+            /* avoid deadlocks (psr_write triggers callbacks that may lock this mutex) */
+            bd_mutex_lock(&bd->mutex);
+        }
+        int res = bd_psr_write(bd->regs, reg, value);
+        if (psr < 102) {
+            bd_mutex_unlock(&bd->mutex);
+        }
+        return res;
+    } else {
+        return bd_gpr_write(bd->regs, reg, value);
+    }
+}
+#endif
+
 #ifdef USING_BDJAVA
 /*
  * handle graphics updates from BD-J layer
@@ -994,7 +1090,11 @@ static void _bdj_osd_cb(BLURAY *bd, const unsigned *img, int w, int h,
     }
 
     /* pass only changed region */
-    aov.argb   = img + x0 + y0 * w;
+    if (bd->argb_buffer && (bd->argb_buffer->width < w || bd->argb_buffer->height < h)) {
+        aov.argb   = img;
+    } else {
+        aov.argb   = img + x0 + y0 * w;
+    }
     aov.stride = w;
     aov.x      = x0;
     aov.y      = y0;
@@ -1031,7 +1131,7 @@ static int _start_bdj(BLURAY *bd, unsigned title)
 {
 #ifdef USING_BDJAVA
     if (bd->bdjava == NULL) {
-        bd->bdjava = bdj_open(bd->device_path, bd, bd->regs, bd->index, _bdj_osd_cb, bd->argb_buffer);
+        bd->bdjava = bdj_open(bd->device_path, bd, bd->index, _bdj_osd_cb, bd->argb_buffer);
         if (!bd->bdjava) {
             return 0;
         }
@@ -1573,7 +1673,7 @@ static int _bd_read(BLURAY *bd, unsigned char *buf, int len)
                 // We previously reached the last clip.  Nothing
                 // else to read.
                 _queue_event(bd, BD_EVENT_END_OF_TITLE, 0);
-                _bdj_event(bd, BDJ_EVENT_END_OF_PLAYLIST, 0);
+                bd->bdj_end_of_playlist |= 1;
                 return 0;
             }
             if (st->int_buf_off == 6144 || clip_pkt >= st->clip->end_pkt) {
@@ -1605,7 +1705,7 @@ static int _bd_read(BLURAY *bd, unsigned char *buf, int len)
                     if (st->clip == NULL) {
                         BD_DEBUG(DBG_BLURAY | DBG_STREAM, "End of title\n");
                         _queue_event(bd, BD_EVENT_END_OF_TITLE, 0);
-                        _bdj_event(bd, BDJ_EVENT_END_OF_PLAYLIST, 0);
+                        bd->bdj_end_of_playlist |= 1;
                         return 0;
                     }
                     if (!_open_m2ts(bd, st)) {
@@ -1916,6 +2016,7 @@ static int _open_playlist(BLURAY *bd, const char *f_name, unsigned angle)
 
     bd->seamless_angle_change = 0;
     bd->s_pos = 0;
+    bd->bdj_end_of_playlist = 0;
 
     bd_psr_write(bd->regs, PSR_PLAYLIST, atoi(bd->title->name));
     bd_psr_write(bd->regs, PSR_ANGLE_NUMBER, bd->title->angle + 1);
@@ -2082,9 +2183,28 @@ uint32_t bd_get_titles(BLURAY *bd, uint8_t flags, uint32_t min_title_length)
         return 0;
     }
 
+    /* start BD+. No real title info will be passed to BD+ VM ... */
+    if (bd->libbdplus) {
+        libbdplus_event(bd->libbdplus, 0xffffffff, bd->index->num_titles, 0);
+    }
+
     return bd->title_list->count;
 }
 
+int bd_get_main_title(BLURAY *bd)
+{
+    if (bd->title_type != title_undef) {
+        BD_DEBUG(DBG_CRIT | DBG_BLURAY, "bd_get_main_title() can't be used with BluRay menus\n");
+    }
+
+    if (bd->title_list == NULL) {
+        BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Title list not yet read!\n");
+        return -1;
+    }
+
+    return bd->title_list->main_title_idx;
+}
+
 static void _copy_streams(NAV_CLIP *clip, BLURAY_STREAM_INFO *streams, MPLS_STREAM *si, int count)
 {
     int ii;
@@ -2434,6 +2554,7 @@ static void _process_psr_write_event(BLURAY *bd, BD_PSR_EVENT *ev)
             break;
         case PSR_TITLE_NUMBER:
             _queue_event(bd, BD_EVENT_TITLE,    ev->new_val);
+            libbdplus_event(bd->libbdplus, 0x110, ev->new_val, 0);
             break;
         case PSR_PLAYLIST:
             _queue_event(bd, BD_EVENT_PLAYLIST, ev->new_val);
@@ -2569,9 +2690,22 @@ static void _queue_initial_psr_events(BLURAY *bd)
 
 static int _play_bdj(BLURAY *bd, unsigned title)
 {
+    if (!bd->disc_info.bdj_handled) {
+        BD_DEBUG(DBG_BLURAY | DBG_CRIT, "Can't play BD-J title %d\n", title);
+        return 0;
+    }
+
+    int result;
+
     bd->title_type = title_bdj;
 
-    return _start_bdj(bd, title);
+    result = _start_bdj(bd, title);
+    if (result <= 0) {
+        bd->title_type = title_undef;
+        _queue_event(bd, BD_EVENT_ERROR, BD_ERROR_BDJ);
+    }
+
+    return result;
 }
 
 static int _play_hdmv(BLURAY *bd, unsigned id_ref)
@@ -2592,6 +2726,11 @@ static int _play_hdmv(BLURAY *bd, unsigned id_ref)
 
     bd->hdmv_suspended = !hdmv_vm_running(bd->hdmv_vm);
 
+    if (result <= 0) {
+        bd->title_type = title_undef;
+        _queue_event(bd, BD_EVENT_ERROR, BD_ERROR_HDMV);
+    }
+
     return result;
 }
 
@@ -2696,6 +2835,9 @@ int bd_play(BLURAY *bd)
     _queue_initial_psr_events(bd);
     bd_psr_unlock(bd->regs);
 
+    /* start BD+ VM */
+    libbdplus_start(bd->libbdplus);
+
     /* start playback from FIRST PLAY title */
 
     result = _play_title(bd, BLURAY_TITLE_FIRST_PLAY);
@@ -2862,7 +3004,7 @@ static int _run_hdmv(BLURAY *bd)
 
     /* run VM */
     if (hdmv_vm_run(bd->hdmv_vm, &hdmv_ev) < 0) {
-        _queue_event(bd, BD_EVENT_ERROR, 0);
+        _queue_event(bd, BD_EVENT_ERROR, BD_ERROR_HDMV);
         bd->hdmv_suspended = !hdmv_vm_running(bd->hdmv_vm);
         return -1;
     }
@@ -2910,6 +3052,11 @@ static int _read_ext(BLURAY *bd, unsigned char *buf, int len, BD_EVENT *event)
         return 0;
     }
 
+    if (bd->bdj_end_of_playlist == 1) {
+        _bdj_event(bd, BDJ_EVENT_END_OF_PLAYLIST, 0);
+        bd->bdj_end_of_playlist |= 2;
+    }
+
     if (!bd->title && bd->title_type == title_bdj) {
         /* BD-J title running but no playlist playing */
         _queue_event(bd, BD_EVENT_IDLE, 0);
