@@ -80,6 +80,9 @@ struct graphics_controller_s {
     BD_IG_EFFECT_SEQUENCE *out_effects;
     int64_t                next_effect_time; /* 90 kHz */
 
+    /* timers */
+    int64_t                user_timeout;
+
     /* animated buttons */
     unsigned               frame_interval;
     unsigned               button_effect_running;
@@ -327,6 +330,20 @@ static uint16_t _find_selected_button_id(GRAPHICS_CONTROLLER *gc)
     return 0xffff;
 }
 
+static void _reset_user_timeout(GRAPHICS_CONTROLLER *gc)
+{
+    gc->user_timeout = 0;
+
+    if (gc->igs->ics->interactive_composition.ui_model == IG_UI_MODEL_POPUP ||
+        bd_psr_read(gc->regs, PSR_MENU_PAGE_ID) != 0) {
+
+        gc->user_timeout = gc->igs->ics->interactive_composition.user_timeout_duration;
+        if (gc->user_timeout) {
+            gc->user_timeout += bd_get_scr();
+        }
+    }
+}
+
 static int _save_page_state(GRAPHICS_CONTROLLER *gc)
 {
     if (!gc->bog_data) {
@@ -418,6 +435,9 @@ static void _reset_page_state(GRAPHICS_CONTROLLER *gc)
     gc->effect_idx  = 0;
     gc->in_effects  = NULL;
     gc->out_effects = NULL;
+
+    /* timers */
+    _reset_user_timeout(gc);
 }
 
 /*
@@ -575,6 +595,7 @@ static void _render_composition_object(GRAPHICS_CONTROLLER *gc,
                                        int palette_update_flag)
 {
     if (gc->overlay_proc) {
+        BD_PG_RLE_ELEM *cropped_img = NULL;
         BD_OVERLAY ov = {0};
         ov.cmd     = BD_OVERLAY_DRAW;
         ov.pts     = pts;
@@ -585,20 +606,21 @@ static void _render_composition_object(GRAPHICS_CONTROLLER *gc,
         ov.h       = object->height;
         ov.palette = palette->entry;
         ov.img     = object->img;
+
         if (cobj->crop_flag) {
-            ov.crop_x  = cobj->crop_x;
-            ov.crop_y  = cobj->crop_y;
-            ov.crop_w  = cobj->crop_w;
-            ov.crop_h  = cobj->crop_h;
-        }
-        if (ov.crop_h) {
-          ov.h = ov.crop_h;
-          ov.crop_h = 0;
+            if (cobj->crop_x || cobj->crop_y || cobj->crop_w != object->width) {
+                ov.img = cropped_img = rle_crop_object(object->img, object->width,
+                                                       cobj->crop_x, cobj->crop_y, cobj->crop_w, cobj->crop_h);
+            }
+            ov.w  = cobj->crop_w;
+            ov.h  = cobj->crop_h;
         }
 
         ov.palette_update_flag = palette_update_flag;
 
         gc->overlay_proc(gc->overlay_proc_handle, &ov);
+
+        bd_refcnt_dec(cropped_img);
     }
 }
 
@@ -829,7 +851,8 @@ int gc_decode_ts(GRAPHICS_CONTROLLER *gc, uint16_t pid, uint8_t *block, unsigned
                 GC_TRACE("gc_decode_ts(): IG selection_timeout_pts not implemented\n");
             }
             if (gc->igs->ics->interactive_composition.user_timeout_duration) {
-                GC_TRACE("gc_decode_ts(): IG user_timeout_duration not implemented\n");
+                GC_TRACE("gc_decode_ts(): IG user_timeout_duration %d\n",
+                         gc->igs->ics->interactive_composition.user_timeout_duration);
             }
         }
 
@@ -925,7 +948,7 @@ static int _render_textst_region(GRAPHICS_CONTROLLER *p, int64_t pts, BD_TEXTST_
         rle_add_eol(&rle);
     }
 
-    _render_rle(p, pts, rle.start,
+    _render_rle(p, pts, rle_get(&rle),
                 style->region_info.region.xpos, style->region_info.region.ypos,
                 style->region_info.region.width, style->region_info.region.height,
                 palette);
@@ -1026,6 +1049,8 @@ static int _render_textst(GRAPHICS_CONTROLLER *p, uint32_t stc, GC_NAV_CMDS *cmd
             textst_render(p->textst_render, &bmp, style, region);
 
             _render_textst_region(p, dialog[ii].start_pts, style, &bmp, s->style->palette);
+
+            X_FREE(bmp.mem);
         }
 
         /* commit changes */
@@ -1282,6 +1307,8 @@ static int _render_effect(GRAPHICS_CONTROLLER *gc, BD_IG_EFFECT *effect)
 
     _flush_osd(gc, BD_OVERLAY_IG, pts);
 
+    _reset_user_timeout(gc);
+
     return 0;
 }
 
@@ -1450,6 +1477,9 @@ static int _user_input(GRAPHICS_CONTROLLER *gc, uint32_t key, GC_NAV_CMDS *cmds)
         GC_ERROR("_user_input(): menu not visible\n");
         return 0;
     }
+
+    _reset_user_timeout(gc);
+
     if (gc->button_effect_running) {
         GC_ERROR("_user_input(): button_effect_running\n");
         return 0;
@@ -1749,6 +1779,8 @@ static int _mouse_move(GRAPHICS_CONTROLLER *gc, uint16_t x, uint16_t y, GC_NAV_C
         _select_button(gc, new_btn_id);
 
         _render_page(gc, -1, cmds);
+
+        _reset_user_timeout(gc);
     }
 
     return gc->valid_mouse_position;
@@ -1789,7 +1821,7 @@ static int _animate(GRAPHICS_CONTROLLER *gc, GC_NAV_CMDS *cmds)
                 result = _render_page(gc, 0xffff, cmds);
             }
 
-        } else {
+        } else if (gc->button_animation_running) {
             int64_t pts = bd_get_scr();
             if (pts >= (gc->next_effect_time + gc->frame_interval)) {
                 gc->next_effect_time += gc->frame_interval;
@@ -1801,6 +1833,38 @@ static int _animate(GRAPHICS_CONTROLLER *gc, GC_NAV_CMDS *cmds)
     return result;
 }
 
+static int _run_timers(GRAPHICS_CONTROLLER *gc, GC_NAV_CMDS *cmds)
+{
+    int result = -1;
+
+    if (gc->ig_open) {
+
+        result = 0;
+
+        if (gc->user_timeout) {
+            int64_t pts = bd_get_scr();
+            if (pts > gc->user_timeout) {
+
+                GC_TRACE("user timeout expired\n");
+
+                if (gc->igs->ics->interactive_composition.ui_model != IG_UI_MODEL_POPUP) {
+
+                    if (bd_psr_read(gc->regs, PSR_MENU_PAGE_ID) != 0) {
+                        _select_page(gc, 0, 0);
+                        result = _render_page(gc, 0xffff, cmds);
+                    }
+
+                } else {
+                    gc->popup_visible = 0;
+                    result = _render_page(gc, 0xffff, cmds);
+                }
+            }
+        }
+    }
+
+    return result;
+}
+
 int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS *cmds)
 {
     int result = -1;
@@ -1896,6 +1960,7 @@ int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS
 
         case GC_CTRL_NOP:
             result = _animate(gc, cmds);
+            _run_timers(gc, cmds);
             break;
 
         case GC_CTRL_INIT_MENU:
@@ -1936,7 +2001,7 @@ int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS
         if (gc->ig_drawn) {
             cmds->status |= GC_STATUS_MENU_OPEN;
         }
-        if (gc->in_effects || gc->out_effects || gc->button_animation_running) {
+        if (gc->in_effects || gc->out_effects || gc->button_animation_running || gc->user_timeout) {
             /* do not trigger if unopened pop-up menu has animations */
             if (gc->ig_open) {
                 cmds->status |= GC_STATUS_ANIMATE;
