@@ -60,12 +60,17 @@ typedef enum {
     GC_CTRL_PG_UPDATE,       /* render decoded PG composition */
     GC_CTRL_PG_RESET,        /* reset PG composition state */
 
+    /* TextST */
+    GC_CTRL_PG_CHARCODE,
+    GC_CTRL_STYLE_SELECT,    /* select next TextST user style */
+
 } gc_ctrl_e;
 
 
 #define GC_STATUS_NONE      0
 #define GC_STATUS_POPUP     1  /* popup menu loaded */
 #define GC_STATUS_MENU_OPEN 2  /* menu open */
+#define GC_STATUS_ANIMATE   4  /* animation or effect running */
 
 typedef struct {
     /* HDMV navigation command sequence */
@@ -77,6 +82,9 @@ typedef struct {
 
     /* graphics status (none, menu, popup) */
     uint32_t status; /* bit mask */
+
+    /* */
+    uint32_t wakeup_time;
 } GC_NAV_CMDS;
 
 /*
@@ -113,4 +121,11 @@ BD_PRIVATE int                  gc_run(GRAPHICS_CONTROLLER *p,
                                        /* out */ GC_NAV_CMDS *cmds);
 
 
+/*
+ * Add TextST font
+ */
+
+BD_PRIVATE int                  gc_add_font(GRAPHICS_CONTROLLER *p,
+                                            const char *font_file);
+
 #endif // _GRAPHICS_CONTROLLER_H_
