@@ -84,27 +84,43 @@ static int _add_ts(PES_BUFFER *p, unsigned pusi, uint8_t *buf, unsigned len)
     int result = 0;
 
     if (pusi) {
-        // Parse PES header
-        unsigned pes_length = buf[4] << 8 | buf[5];
-        unsigned pts_exists = buf[7] & 0x80;
-        unsigned dts_exists = buf[7] & 0x40;
-        unsigned hdr_len    = buf[8] + 9;
 
+        if (len < 6) {
+            BD_DEBUG(DBG_DECODE, "invalid BDAV TS (PES header not in single TS packet)\n");
+            return -1;
+        }
         if (buf[0] || buf[1] || buf[2] != 1) {
             BD_DEBUG(DBG_DECODE, "invalid PES header (00 00 01)");
             return -1;
         }
 
-        if (len < hdr_len) {
-            BD_DEBUG(DBG_DECODE, "invalid BDAV TS (PES header not in single TS packet)\n");
-            return -1;
-        }
+        // Parse PES header
+        unsigned pes_pid    = buf[3];
+        unsigned pes_length = buf[4] << 8 | buf[5];
+        unsigned hdr_len    = 6;
 
-        if (pts_exists) {
-            p->pts = _parse_timestamp(buf + 9);
-        }
-        if (dts_exists) {
-            p->dts = _parse_timestamp(buf + 14);
+        if (pes_pid != 0xbf) {
+
+            if (len < 9) {
+                BD_DEBUG(DBG_DECODE, "invalid BDAV TS (PES header not in single TS packet)\n");
+                return -1;
+            }
+
+            unsigned pts_exists = buf[7] & 0x80;
+            unsigned dts_exists = buf[7] & 0x40;
+            hdr_len += buf[8] + 3;
+
+            if (len < hdr_len) {
+                BD_DEBUG(DBG_DECODE, "invalid BDAV TS (PES header not in single TS packet)\n");
+                return -1;
+            }
+
+            if (pts_exists) {
+                p->pts = _parse_timestamp(buf + 9);
+            }
+            if (dts_exists) {
+                p->dts = _parse_timestamp(buf + 14);
+            }
         }
 
         buf += hdr_len;
@@ -116,7 +132,8 @@ static int _add_ts(PES_BUFFER *p, unsigned pusi, uint8_t *buf, unsigned len)
     // realloc
     if (p->size < p->len + len) {
         p->size *= 2;
-        p->buf   = realloc(p->buf, p->size);
+        p->size = BD_MAX(p->size, BD_MAX(result, 0x100));
+        p->buf  = realloc(p->buf, p->size);
     }
 
     // append
@@ -173,7 +190,7 @@ PES_BUFFER *m2ts_demux(M2TS_DEMUX *p, uint8_t *buf)
                       p->buf->len, p->pes_length);
                 pes_buffer_free(&p->buf);
             }
-            p->buf = pes_buffer_alloc(0xffff);
+            p->buf = pes_buffer_alloc();
         }
 
         if (!p->buf) {
